@@ -38,7 +38,7 @@ dbus_register_service (DBusGProxy  *proxy,
 	GError *error = NULL;
 	guint	result;
 
-	g_message ("Registering DBus service...\n"
+	g_message ("Registering D-Bus service...\n"
 		   "  Name:'%s'",
 		   name);
 
@@ -55,7 +55,7 @@ dbus_register_service (DBusGProxy  *proxy,
 	}
 
 	if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-		g_critical ("DBus service name:'%s' is already taken, "
+		g_critical ("D-Bus service name:'%s' is already taken, "
 			    "perhaps the daemon is already running?",
 			    name);
 		return FALSE;
@@ -71,7 +71,7 @@ dbus_register_object (DBusGConnection	    *lconnection,
 		      const DBusGObjectInfo *info,
 		      const gchar	    *path)
 {
-	g_message ("Registering DBus object...");
+	g_message ("Registering D-Bus object...");
 	g_message ("  Path:'%s'", path);
 	g_message ("  Type:'%s'", G_OBJECT_TYPE_NAME (object));
 
@@ -97,7 +97,7 @@ dbus_register_names (void)
 	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 
 	if (!connection) {
-		g_critical ("Could not connect to the DBus session bus, %s",
+		g_critical ("Could not connect to the D-Bus session bus, %s",
 			    error ? error->message : "no error given.");
 		g_clear_error (&error);
 		return FALSE;
@@ -111,7 +111,7 @@ dbus_register_names (void)
 					    DBUS_PATH_DBUS,
 					    DBUS_INTERFACE_DBUS);
 
-	/* Register the service name for org.freedesktop.Tracker.Extract */
+	/* Register the service name for org.freedesktop.Tracker1.Extract */
 	if (!dbus_register_service (gproxy, TRACKER_EXTRACT_SERVICE)) {
 		return FALSE;
 	}
@@ -158,11 +158,11 @@ tracker_dbus_register_objects (void)
 	gpointer object;
 
 	if (!connection || !gproxy) {
-		g_critical ("DBus support must be initialized before registering objects!");
+		g_critical ("D-Bus support must be initialized before registering objects!");
 		return FALSE;
 	}
 
-	/* Add org.freedesktop.Tracker.Extract */
+	/* Add org.freedesktop.Tracker1.Extract */
 	object = tracker_extract_new ();
 	if (!object) {
 		g_critical ("Could not create TrackerExtract object to register");
