@@ -25,9 +25,17 @@
 #include <glib.h>
 
 #include <libtracker-common/tracker-os-dependant.h>
+#include <libtracker-common/tracker-statement-list.h>
+#include <libtracker-common/tracker-ontology.h>
 
 #include "tracker-main.h"
-#include "tracker-escape.h"
+
+#define NIE_PREFIX TRACKER_NIE_PREFIX
+#define NFO_PREFIX TRACKER_NFO_PREFIX
+#define NCO_PREFIX TRACKER_NCO_PREFIX
+
+#define RDF_PREFIX TRACKER_RDF_PREFIX
+#define RDF_TYPE RDF_PREFIX "type"
 
 typedef enum {
 	READ_TITLE,
@@ -37,12 +45,13 @@ typedef enum {
 	READ_COMMENTS,
 	READ_STATS,
 	READ_CREATED,
-	READ_FILE_OTHER
+	READ_GENERATOR
 } tag_type;
 
 typedef struct {
-	GHashTable *metadata;
+	TrackerSparqlBuilder *metadata;
 	tag_type current;
+	const gchar *uri;
 } ODTParseInfo;
 
 static void start_element_handler (GMarkupParseContext	*context,
@@ -61,7 +70,7 @@ static void text_handler	  (GMarkupParseContext	*context,
 				   gpointer		 user_data,
 				   GError	       **error);
 static void extract_oasis	  (const gchar		*filename,
-				   GHashTable		*metadata);
+				   TrackerSparqlBuilder		*metadata);
 
 static TrackerExtractData extract_data[] = {
 	{ "application/vnd.oasis.opendocument.*", extract_oasis },
@@ -69,19 +78,21 @@ static TrackerExtractData extract_data[] = {
 };
 
 static void
-extract_oasis (const gchar *filename,
-	       GHashTable  *metadata)
+extract_oasis (const gchar *uri,
+	       TrackerSparqlBuilder   *metadata)
 {
 	gchar	      *argv[5];
 	gchar	      *xml;
+	gchar *filename = g_filename_from_uri (uri, NULL, NULL);
 	ODTParseInfo   info = {
 		metadata,
-		-1
+		-1,
+		uri
 	};
 
 	argv[0] = g_strdup ("unzip");
 	argv[1] = g_strdup ("-p");
-	argv[2] = g_strdup (filename);
+	argv[2] = filename;
 	argv[3] = g_strdup ("meta.xml");
 	argv[4] = NULL;
 
@@ -95,6 +106,10 @@ extract_oasis (const gchar *filename,
 			NULL
 		};
 
+		tracker_statement_list_insert (metadata, uri, 
+		                          RDF_TYPE, 
+		                          NFO_PREFIX "PaginatedTextDocument");
+
 		context = g_markup_parse_context_new (&parser, 0, &info, NULL);
 		g_markup_parse_context_parse (context, xml, -1, NULL);
 
@@ -103,9 +118,10 @@ extract_oasis (const gchar *filename,
 	}
 
 	g_free (argv[3]);
-	g_free (argv[2]);
 	g_free (argv[1]);
 	g_free (argv[0]);
+
+	g_free (filename);
 }
 
 void
@@ -134,21 +150,23 @@ start_element_handler (GMarkupParseContext  *context,
 		data->current = READ_COMMENTS;
 	}
 	else if (strcmp (element_name, "meta:document-statistic") == 0) {
-		GHashTable *metadata;
+		TrackerSparqlBuilder *metadata;
+		const gchar *uri;
 		const gchar **a, **v;
 
 		metadata = data->metadata;
+		uri = data->uri;
 
 		for (a = attribute_names, v = attribute_values; *a; ++a, ++v) {
 			if (strcmp (*a, "meta:word-count") == 0) {
-				g_hash_table_insert (metadata,
-						     g_strdup ("Doc:WordCount"),
-						     tracker_escape_metadata (*v));
+				tracker_statement_list_insert (metadata, uri,
+							  NFO_PREFIX "wordCount",
+							  *v);
 			}
 			else if (strcmp (*a, "meta:page-count") == 0) {
-				g_hash_table_insert (metadata,
-						     g_strdup ("Doc:PageCount"),
-						     tracker_escape_metadata (*v));
+				tracker_statement_list_insert (metadata, uri,
+							  NFO_PREFIX "pageCount",
+							  *v);
 			}
 		}
 
@@ -158,7 +176,7 @@ start_element_handler (GMarkupParseContext  *context,
 		data->current = READ_CREATED;
 	}
 	else if (strcmp (element_name, "meta:generator") == 0) {
-		data->current = READ_FILE_OTHER;
+		data->current = READ_GENERATOR;
 	}
 	else {
 		data->current = -1;
@@ -182,59 +200,55 @@ text_handler (GMarkupParseContext  *context,
 	      GError		  **error)
 {
 	ODTParseInfo *data;
-	GHashTable   *metadata;
+	TrackerSparqlBuilder    *metadata;
+	const gchar        *uri;
 
 	data = user_data;
 	metadata = data->metadata;
+	uri = data->uri;
 
 	switch (data->current) {
 	case READ_TITLE:
-		g_hash_table_insert (metadata,
-				     g_strdup ("Doc:Title"),
-				     tracker_escape_metadata (text));
+		tracker_statement_list_insert (metadata, uri,
+					  NIE_PREFIX "title",
+					  text);
 		break;
 	case READ_SUBJECT:
-		g_hash_table_insert (metadata,
-				     g_strdup ("Doc:Subject"),
-				     tracker_escape_metadata (text));
+		tracker_statement_list_insert (metadata, uri,
+					  NIE_PREFIX "subject",
+					  text);
 		break;
 	case READ_AUTHOR:
-		g_hash_table_insert (metadata,
-				     g_strdup ("Doc:Author"),
-				     tracker_escape_metadata (text));
+		tracker_statement_list_insert (metadata, ":", RDF_TYPE, NCO_PREFIX "Contact");
+		tracker_statement_list_insert (metadata, ":", NCO_PREFIX "fullname", text);
+		tracker_statement_list_insert (metadata, uri, NCO_PREFIX "creator", ":");
 		break;
 	case READ_KEYWORDS: {
-		gchar *keywords;
-
-		if ((keywords = g_hash_table_lookup (metadata, "Doc:Keywords"))) {
-			gchar *escaped;
-
-			escaped = tracker_escape_metadata (text);
-			g_hash_table_replace (metadata,
-					      g_strdup ("Doc:Keywords"),
-					      g_strconcat (keywords, ",", escaped, NULL));
-			g_free (escaped);
-		} else {
-			g_hash_table_insert (metadata,
-					     g_strdup ("Doc:Keywords"),
-					     tracker_escape_metadata (text));
+		gchar *keywords = g_strdup (text);
+		char *lasts, *keyw;
+		for (keyw = strtok_r (keywords, ",; ", &lasts); keyw; 
+		     keyw = strtok_r (NULL, ",; ", &lasts)) {
+			tracker_statement_list_insert (metadata,
+					  uri, NIE_PREFIX "keyword",
+					  (const gchar*) keyw);
 		}
+		g_free (keywords);
 	}
 		break;
 	case READ_COMMENTS:
-		g_hash_table_insert (metadata,
-				     g_strdup ("Doc:Comments"),
-				     tracker_escape_metadata (text));
+		tracker_statement_list_insert (metadata, uri,
+					  NIE_PREFIX "comment",
+					  text);
 		break;
 	case READ_CREATED:
-		g_hash_table_insert (metadata,
-				     g_strdup ("Doc:Created"),
-				     tracker_escape_metadata (text));
+		tracker_statement_list_insert (metadata, uri,
+					  NIE_PREFIX "contentCreated",
+					  text);
 		break;
-	case READ_FILE_OTHER:
-		g_hash_table_insert (metadata,
-				     g_strdup ("File:Other"),
-				     tracker_escape_metadata (text));
+	case READ_GENERATOR:
+		tracker_statement_list_insert (metadata, uri,
+					  NIE_PREFIX "generator",
+					  text);
 		break;
 
 	default:
